(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "fs", "path", "./file-utils", "./environment-params"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var fs = require("fs");
    var path = require("path");
    var FileUtils = require("./file-utils");
    var environment_params_1 = require("./environment-params");
    /*
        This class expose an API to read, create and update the configuration of a project.
     */
    var ConfigManager = /** @class */ (function () {
        function ConfigManager(consoleLogger, packageType, environmentParams, logger) {
            this.environmentParams = new environment_params_1.EnvironmentParams();
            this.consoleLogger = consoleLogger;
            this.packageType = packageType;
            this.logger = logger;
        }
        ConfigManager.prototype.read = function () {
            var fileName = this.getFileName();
            try {
                var hasConfigFile = FileUtils.isFileExists(fileName);
                if (hasConfigFile) {
                    return this.readFromConfigFile(fileName);
                }
                var config = this.readFromPackageJson();
                return config;
            }
            catch (err) {
                this.consoleLogger.printError("Failed to read configuration for " + this.packageType + ". error: " + err);
                throw err;
            }
        };
        ConfigManager.prototype.readFromConfigFile = function (fileName) {
            var filePath = path.join(this.environmentParams.getBasePath(), fileName);
            try {
                var content = FileUtils.readJsonFileSafe(filePath, this.consoleLogger);
                return content;
            }
            catch (err) {
                throw err;
            }
        };
        ConfigManager.prototype.readFromPackageJson = function () {
            var packagePath = path.join(this.environmentParams.getBasePath(), "package.json");
            var configKey = this.getConfigKey();
            try {
                var content = FileUtils.readJsonFileSafe(packagePath, this.consoleLogger);
                if (!content[configKey]) {
                    this.consoleLogger.println("Configuration for " + this.packageType + " not found in package.json");
                    return null;
                }
                return content[configKey];
            }
            catch (err) {
                this.consoleLogger.printError("Failed to read " + configKey + " from package.json.");
                throw err;
            }
        };
        ConfigManager.prototype.write = function (config) {
            var fileName = this.getFileName();
            var hasConfigFile = this.hasConfigFile(fileName);
            if (hasConfigFile) {
                return this.writeToConfigFile(config);
            }
            return this.writeToPackageJson(config);
        };
        ;
        ConfigManager.prototype.writeToConfigFile = function (config) {
            var fileName = this.getFileName();
            var filePath = path.resolve(this.environmentParams.getBasePath(), fileName);
            try {
                FileUtils.writeJsonFileSafe(filePath, config, this.consoleLogger);
                return true;
            }
            catch (err) {
                this.consoleLogger.printError("Error while trying to write to " + this.packageType + " config file");
                return false;
            }
        };
        ConfigManager.prototype.writeToPackageJson = function (config) {
            var packagePath = path.join(this.environmentParams.getBasePath(), "package.json");
            var configKey = this.getConfigKey();
            try {
                var content = FileUtils.readJsonFileSafe(packagePath, this.consoleLogger);
                if (content)
                    content[configKey] = config;
                FileUtils.writeJsonFileSafe(packagePath, config, this.consoleLogger);
                return true;
            }
            catch (err) {
                this.consoleLogger.printError("Error while trying write to package.json");
                return false;
            }
        };
        ConfigManager.prototype.saveOriginalConfig = function (config) {
            if (!config) {
                return;
            }
            var backupFile = this.getFileName() + ".slback";
            this.consoleLogger.println("saving original " + this.packageType + " config to '" + backupFile + "'");
            var filePath = path.join(this.environmentParams.getBasePath(), backupFile);
            try {
                FileUtils.writeJsonFileSafe(filePath, config, this.consoleLogger);
            }
            catch (err) {
                this.consoleLogger.printError("Error while saving original file");
            }
        };
        ConfigManager.prototype.hasConfigFile = function (fileName) {
            var configFile = path.join(this.environmentParams.getBasePath(), fileName);
            if (fs.existsSync(configFile)) {
                return true;
            }
            return false;
        };
        return ConfigManager;
    }());
    exports.ConfigManager = ConfigManager;
    var Packages;
    (function (Packages) {
        Packages["BABEL"] = "babel";
        Packages["NYC"] = "nyc";
    })(Packages = exports.Packages || (exports.Packages = {}));
});
//# sourceMappingURL=config-manager.js.map