(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "path", "fs", "./environment-params", "rimraf", "http", "https", "./logger"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var path = require("path");
    var fs = require("fs");
    var environment_params_1 = require("./environment-params");
    var rimraf = require("rimraf");
    var http = require("http");
    var https = require("https");
    var logger_1 = require("./logger");
    /*
        This class handle all file operations in safe way.
     */
    var logger = logger_1.LoggerFactory.getCreateApplicationLogger();
    function readJsonFileSafe(filePath, consoleLogger) {
        try {
            var file = fs.readFileSync(filePath);
            var originalValue = file.toString();
            var jsonObject = tryParseToJson(originalValue, consoleLogger, filePath);
            return jsonObject;
        }
        catch (err) {
            consoleLogger.printError("Error while trying to read: " + filePath + " " + err);
            throw err;
        }
    }
    exports.readJsonFileSafe = readJsonFileSafe;
    function writeFileSafe(filePath, content, consoleLogger) {
        try {
            fs.writeFileSync(filePath, content);
        }
        catch (err) {
            consoleLogger.printError("Error while trying to write File: " + filePath + " " + err);
            throw err;
        }
    }
    exports.writeFileSafe = writeFileSafe;
    function writeJsonFileSafe(filePath, config, consoleLogger) {
        try {
            var content = JSON.stringify(config, null, "\t");
            writeFileSafe(filePath, content, consoleLogger);
        }
        catch (err) {
            consoleLogger.printError("Error while trying to write File: " + filePath + " " + err);
            throw err;
        }
    }
    exports.writeJsonFileSafe = writeJsonFileSafe;
    function isFileExists(fileName) {
        var environmentParams = new environment_params_1.EnvironmentParams();
        var file = path.resolve(environmentParams.getBasePath(), fileName);
        return fs.existsSync(file);
    }
    exports.isFileExists = isFileExists;
    var tryParseToJson = function (originalValue, consoleLogger, filePath) {
        // Add quotes to all keys for parsing.
        var valid = originalValue.replace((/^[ ]*(\w+)(:)/gm), "\"$1\"$2");
        try {
            return JSON.parse(valid);
        }
        catch (err) {
            consoleLogger.printError("Error while trying to parse " + filePath + " " + err.message);
            throw err;
        }
    };
    function findFileInFolder(filename, folder, logger) {
        var filePath = path.resolve(folder, filename);
        if (fs.existsSync(filePath)) {
            logger.info("Found file '" + filename + "'  in: '" + folder + "'");
            return filePath;
        }
        return null;
    }
    exports.findFileInFolder = findFileInFolder;
    function downloadFile(url, dest, logger) {
        var httplib = (url.indexOf('https') == 0) ? https : http;
        return new Promise(function (resolve, reject) {
            var file = fs.createWriteStream(dest);
            file.on("error", function (err) {
                logger.warn("Error while trying to write file '" + dest + "' , Error: ", err);
                reject(err);
            });
            file.on('close', function () {
                logger.info("file '" + dest + "' downloaded successfully.");
                resolve();
            });
            httplib.get(url, function (response) {
                response.pipe(file);
            }).on('error', function (err) {
                logger.warn("Error while trying to download file " + dest + ", Error: ", err);
                logger.warn("Deleting file " + dest);
                fs.unlink(dest, function () {
                });
                reject(err);
            });
        });
    }
    exports.downloadFile = downloadFile;
    ;
    function createMultipleDirs(dir) {
        if (fs.existsSync(dir)) {
            if (fs.statSync(dir).isDirectory()) {
                return;
            }
            throw new Error("Not a directory: ," + dir);
        }
        var i = dir.lastIndexOf(path.sep);
        if (i < 0) {
            throw new Error("No parent directory: " + dir);
        }
        createMultipleDirs(dir.substring(0, i));
        fs.mkdirSync(dir);
    }
    exports.createMultipleDirs = createMultipleDirs;
    function renameFolder(currentName, newName) {
        validateFolder(currentName);
        fs.renameSync(currentName, newName);
        logger.info("Folder '" + currentName + "' was renamed to '" + newName);
    }
    exports.renameFolder = renameFolder;
    function deleteFolder(folder) {
        validateFolder(folder);
        rimraf.sync(folder);
        logger.info("Folder '" + folder + "' was deleted successfully");
    }
    exports.deleteFolder = deleteFolder;
    function validateFolder(folder) {
        var folderStat = fs.statSync(folder);
        if (!folderStat.isDirectory()) {
            throw new Error("'" + folder + "' is not directory");
        }
    }
});
//# sourceMappingURL=file-utils.js.map